#!/bin/sh
#
# dhclient-script: Network interface configuration script run by
#                  dhclient based on DHCP client communication
#
# Copyright (C) 2008, 2009, 2010  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author(s): David Cantrell <dcantrell@redhat.com>
#
# ----------
# This script is a rewrite/reworking on dhclient-script originally
# included as part of dhcp-970306:
# dhclient-script for Linux. Dan Halbert, March, 1997.
# Updated for Linux 2.[12] by Brian J. Murrell, January 1999.
# Modified by David Cantrell <dcantrell@redhat.com> for Fedora and RHEL
# http://pkgs.fedoraproject.org/gitweb/?p=dhcp.git;a=commitdiff;h=fe8e8224b5cbd10550a02c6427f388cc34b4274e#patch2
# Siemens Enterprise Sven Jakob: make script busybox v1.15.3 compliant
# ----------
#
# CONF_DIR=/data/networking
# CONF_DIR_TMP=/tmp/networking
# SAVE_DIR=${CONF_DIR}/dhclient
#
# Files in CONF_DIR_TMP
# dhcpv4.eth0.env     ... dhcpv4 assigned values, written by dhcpv4Event.sh and read/removed by NM
#                         Lease reuse: is a copy from dhcpv4.eth0.env.backup or dhcpv4.eth0.env.vlan.backup
# netv4.env           ... manual assigned values, written by WNI used by netv4.sh
# dhcpv4.eth0.status  ... information how the IPv4 IFC is finally configured => syslog
# portDescr           ... not supported, can give a hint if phone is plugged to a different access switch port (PORT_TYPE_MAC_ADDRESS)
# vlan                ... assigned vlan-id, finally saved in *.env.vlan.backup
# keepOldIpv4Address  ... used to prevent a flush of the IFC, e.g. if it contains "keep_old_ip=yes", flush in PREINIT is skipped
# addressWatchDog.pid ... file created if addressWatchDog is running (only if enableDhcpv4LeaseReuse)
# dhcpv4.eth0.env.backup      ... backup of dhcpv4.eth0.env used for dhcp lease reuse from untagged vlan, stored in normal mode
# dhcpv4.eth0.env.vlan.backup ... backup of dhcpv4.eth0.env used for dhcp lease reuse from tagged vlan, stored in normal mode
#
# Files in CONF_DIR (persistent):
# dhcpv4.eth0.env.backup      ... backup of dhcpv4.eth0.env used for dhcp lease reuse from untagged vlan
# dhcpv4.eth0.env.vlan.backup ... backup of dhcpv4.eth0.env used for dhcp lease reuse from tagged vlan
# portDescr                   ... not supported, can give a hint if phone is plugged to a different access switch port (PORT_TYPE_MAC_ADDRESS)
# enableDhcpv4LeaseReuse      ... if file exists, then the feature is switched on

export PATH=./:/bin:/sbin:/usr/bin:/Opera_Deploy:/data/networking

scriptname=`basename $0`

RM="rm -f"

CONF_DIR=/data/networking
CONF_DIR_TMP=/tmp/networking
SAVE_DIR=${CONF_DIR}/dhclient

if [ ! -d ${CONF_DIR} ]; then
  mkdir -p ${CONF_DIR}
fi

if [ ! -d ${CONF_DIR_TMP} ]; then
  mkdir -p ${CONF_DIR_TMP}
fi

exec 1>>/tmp/logs/${scriptname}.out
exec 2>>/tmp/logs/${scriptname}.out
# set -x # only for debug purpose

LOGFACILITY="daemon"

logmessage()
{
  msg="$1"
  logger -p ${LOGFACILITY}.notice -t "NET [$$]" "dhclient: ${msg}"
}

logmessage_warn()
{
  msg="$1"
  logger -p ${LOGFACILITY}.warning -t "NET [$$]" "dhclient: ${msg}"
}

logmessage_err()
{
  msg="$1"
  logger -p ${LOGFACILITY}.err -t "NET [$$]" "dhclient: ${msg}"
}

if [ -z ${interface} ]; then
  interface=eth0
fi

ETCDIR="/etc"
ETCDIR_DHCP="${ETCDIR}/dhcp"

DHCLIENT_DELAY=1
DHCLIENT_DELAY_DUPLICATE_IP=10
GATEWAYDEV=${interface}
PEERDNS="yes"
SEARCH=""
RES_OPTIONS=""

tmpDhcpParameters=${CONF_DIR_TMP}/dhcpv4.${interface}.env.$$
dhcpv4BackupConfigurationFile=${CONF_DIR}/dhcpv4.${interface}.env.backup
dhcpv4BackupVlanConfigurationFile=${CONF_DIR}/dhcpv4.${interface}.env.vlan.backup
dhcpv4BackupConfigurationTmpFile=${CONF_DIR_TMP}/dhcpv4.${interface}.env.backup
dhcpv4BackupVlanConfigurationTmpFile=${CONF_DIR_TMP}/dhcpv4.${interface}.env.vlan.backup
tmpInterfaceStatus=${CONF_DIR_TMP}/dhcpv4.${interface}.status
tmpVlanFile=${CONF_DIR_TMP}/vlan
tmpKeepOldIpAddress=${CONF_DIR_TMP}/keepOldIpv4Address

# Import common definitions and functions
. /Opera_Deploy/networking/lib.sh

# Dhcp lease reuse configuration
if [ "$1" = "start" ]; then
  # Set environment for configuring the interface
  if [ "$2" ] && [ -r $2 ]; then
    sed -e 's!=!="!' -e 's!$!"!' $2 >${CONF_DIR_TMP}/dhcpv4.backup.$$
    . ${CONF_DIR_TMP}/dhcpv4.backup.$$
    ${RM} ${CONF_DIR_TMP}/dhcpv4.backup.$$
    # Force reason to fully configure ifc
    export reason="REUSE"
    if [ "`ip -4 addr show ${interface} | grep ${new_ip_address}`" ]; then
      keep_old_ip="yes"
    fi
  else
    logmessage_warn "missing configuration for dhcpv4 lease reuse"
  fi
fi

# Prevent read race condition in NetworkManager,
delay_count=5
while [ -f ${tmpDhcpParameters} -a ${delay_count} -gt 0 ]; do
  let delay_count-=1;
  sleep 1
done

if [ ${delay_count} -le 0 ]; then
  # NetworkManager did not remove env file, reason may be a broken communication
  logmessage_warn "WARNING: reason: ${reason}: communication problem to NetworkManager!"
  # Only for debugging
  # cp ${tmpDhcpParameters} ${tmpDhcpParameters}.fatal
  ${RM} ${tmpDhcpParameters}
fi

if [ -r ${tmpKeepOldIpAddress} ]; then
  # Set keep_old_ip to prevent unnecessary flush in case of external forced renew
  . ${tmpKeepOldIpAddress}
fi

# Save temporary the dhclient's environment
if [ ${reason} = "REUSE" ]; then
  sed 's!reason=.*!reason=REUSE!' $2 >${tmpDhcpParameters}
else
  new_host_name=${new_host_name//[^a-zA-Z0-9]/}
  env >${tmpDhcpParameters}
fi

# Only for debugging
# cp ${tmpDhcpParameters} ${tmpDhcpParameters}.${reason}


exit_with_hooks()
{
  exit_status="$1"

  if [ -x ${ETCDIR_DHCP}/dhclient-exit-hooks ]; then
      . ${ETCDIR_DHCP}/dhclient-exit-hooks
  fi

  exit ${exit_status}
}

quad2num()
{
  if [ $# -eq 4 ]; then
    let n="$1 << 24 | $2 << 16 | $3 << 8 | $4"
    echo "${n}"
    return 0
  else
    echo "0"
    return 1
  fi
}

# Result is a decimal number representing the hexadecimal value of the IP address
# $1 = 10.77.68.2 => returns 172835842 (0xA4D4402)
ip2num()
{
  IFS="." quad2num $1
}

# Result is a doted IP address calculated from a decimal number representing the hexadecimal value of the IP address
# $1=172835842 (0xA4D4402) => returns 10.77.68.2
num2ip()
{
  let n="$1"
  let o1="(n >> 24) & 0xff"
  let o2="(n >> 16) & 0xff"
  let o3="(n >> 8) & 0xff"
  let o4="n & 0xff"
  echo "${o1}.${o2}.${o3}.${o4}"
}

# Result is network address by given IP address and network mask or prefix
# e.g.: $1=10.77.68.2, $2=255.255.0.0 => returns 10.77.0.0 (network subnet address)
get_network_address()
{
  ip="$1"
  nm="$2"

  if [ "${ip}" -a "${nm}" ]; then
    if [ `echo "${nm}" | grep '.*\..*'` ]; then
      ipcalc -s -n ${ip} ${nm} | cut -d '=' -f 2
    else
      ipcalc -s -n ${ip}/${nm} | cut -d '=' -f 2
    fi
  fi
}

# Result is prefix by given IP address and network mask
# e.g.: $1=10.77.68.2, $2=255.255.0.0 => returns 16 (result is a prefix)
get_prefix()
{
  ip="$1"
  nm="$2"

  if [ "${ip}" -a "${nm}" ]; then
    ipcalc -s -p ${ip} ${nm} | cut -d '=' -f 2
  fi
}

# Result is the prefix (class bits) from a given (network) address
# $1=129.210.177.0 => returns 24 (result is a prefix)
class_bits()
{
  let ip=$(IFS='.' ip2num $1)

  if [ ${ip} -eq 0 ]; then
    echo 0
    return 0
  fi

  let bits=32
  let mask='255'
  while true; do
    let v='ip&mask'
    if [ "$v" -eq 0 ] ; then
      let bits-=8
    else
      break
    fi
    let 'mask<<=8'
  done
  echo $bits
}

is_router_reachable()
{
  # handle DHCP servers that give us a router not on our subnet
  router="$1"
  routersubnet="$(get_network_address ${router} ${new_subnet_mask})"
  mysubnet="$(get_network_address ${new_ip_address} ${new_subnet_mask})"
  unreachable=0

  if [ ! "${routersubnet}" = "${mysubnet}" ]; then
    unreachable=1
    if arping -f -q -I ${interface} -w 2 ${router}; then
      logmessage "reason: ${reason}: \"ip -4 route add ${router}/32 dev ${interface}\""
      ip -4 route add ${router}/32 dev ${interface}
      if [ $? -eq 0 ]; then
        network_parameter_changed=1
        unreachable=0
      else
        logmessage_err "failed to create host router for unreachable router ${router} not on subnet ${mysubnet}"
      fi
    else
      unreachable=1
      logmessage "DHCP router ${router} is unreachable on DHCP subnet ${mysubnet} router subnet ${routersubnet}"
    fi
  fi

  return ${unreachable}
}

add_default_gateway()
{
  router="$1"
  metric=""

  if [ $# -gt 1 ] && [ $2 -gt 0 ]; then
    metric="metric $2"
  fi

  if is_router_reachable ${router} ; then
    logmessage "reason: ${reason}: is_router_reachable: \"ip -4 route replace default via ${router} dev ${interface} ${metric}\""
    ip -4 route replace default via ${router} dev ${interface} ${metric}
    if [ $? -ne 0 ]; then
      logmessage_err "failed to create default route: ${router} dev ${interface} ${metric}"
      return 1
    else
      network_parameter_changed=1
      return 0
    fi
  fi

  return 1
}


# RFC3442 classless static routes:
# Client MUST zero any bits in the subnet number where the corresponding bit in the mask is zero.
# In other words, the subnet number installed in the routing table is the logical AND of
# the subnet number and subnet mask given in the Classless Static Routes option. (RFC3442)
#
#   Code Len Destination 1    Router 1            Destination 2       Router 2
#  +-----+---+----+-----+----+----+----+----+----+----+-----+----+----+----+----+----+
#  | 121 | n | d1 | ... | dN | r1 | r2 | r3 | r4 | d1 | ... | dN | r1 | r2 | r3 | r4 |
#  +-----+---+----+-----+----+----+----+----+----+----+-----+----+----+----+----+----+
#
#    Width of subnet mask     Number of significant octets
#                 0                     0
#              1- 8                     1
#              9-16                     2
#             17-24                     3
#             25-32                     4
#
#    examples:
#    Subnet number   Subnet mask      Destination descriptor
#    0               0                0
#    10.0.0.0        255.0.0.0        8.10
#    10.0.0.0        255.255.255.0    24.10.0.0
#    10.17.0.0       255.255.0.0      16.10.17
#    10.27.129.0     255.255.255.0    24.10.27.129
#    10.229.0.128    255.255.255.128  25.10.229.0.128
#    10.198.122.47   255.255.255.255  32.10.198.122.47

# Parse and modify the option 121 delivered by dhclient, if defined
# by "option classless-static-routes code 121 = array of unsigned integer 8;"
# e.g.: classless-static-routes=32 172 16 212 3 10 77 68 3 24 172 16 212 10 77 68 3
# returns: a colon-separated list of arguments to pass to "ip route"
modify_classless_static_routes()
{
  route_destination=""
  action=$1
  shift

  while [ $# -ne 0 ]; do
    prefix=$1
    shift

    # Is the destination a multicast group?
    if [ $1 -ge 224 -a $1 -lt 240 ]; then
      multicast=1
    else
      multicast=0
    fi

    # Parse the destination descriptor: "prefix destination"
    if [ ${prefix} -gt 32 ]; then
      logmessage_err "prefix \"${prefix}\" is out of range, see RFC3442 for details"
      break
    elif [ ${prefix} -gt 24 ]; then
      if [ $# -lt 8 ]; then
        logmessage_err "prefix \"${prefix}\" requires 4 byte destination and 4 byte router address, see RFC3442 for details"
        break
      fi
      destination="$1.$2.$3.$4"
      shift; shift; shift; shift
    elif [ ${prefix} -gt 16 ]; then
      if [ $# -lt 7 ]; then
        logmessage_err "prefix \"${prefix}\" requires 3 byte destination and 4 byte router address, see RFC3442 for details"
        break
      fi
      destination="$1.$2.$3.0"
      shift; shift; shift
    elif [ ${prefix} -gt 8 ]; then
      if [ $# -lt 6 ]; then
        logmessage_err "prefix \"${prefix}\" requires 2 byte destination and 4 byte router address, see RFC3442 for details"
        break
      fi
      destination="$1.$2.0.0"
      shift; shift
    elif [ ${prefix} -gt 0 ]; then
      if [ $# -lt 5 ]; then
        logmessage_err "prefix \"${prefix}\" requires 1 byte destination and 4 byte router address, see RFC3442 for details"
        break
      fi
      destination="$1.0.0.0"
      shift
    else # 0.0.0.0/0 is a default route an must be defined as "d1 r1 r2 r3 r4" e.g. in ASCII "0 10.77.68.2"
      if [ $# -lt 4 ]; then
        logmessage_err "prefix \"${prefix}\" requires 0 byte destination and 4 byte router address, see RFC3442 for details"
        break
      fi
      destination="0.0.0.0"
    fi

    # Read the gateway
    gateway="$1.$2.$3.$4"
    shift; shift; shift; shift

    # Multicast routing on Linux
    #  - If you set a next-hop address for a multicast group, this breaks with Cisco switches
    #  - If you simply leave it link-local and attach it to an interface, it works fine.
    if [ ${multicast} -eq 1 ]; then
      route="${destination}/${prefix} dev ${interface}"
    else
        route="${destination}/${prefix} via ${gateway} dev ${interface}"
    fi

    if [ "${action}" = "delete" ]; then
      logmessage "reason: ${reason}: \"ip -4 route delete ${route}\""
      ip -4 route delete ${route}
    else
      metric=''
      for t in ${route_destination}; do
        if [ ${t} = ${destination} ]; then
          if [ -z "${metric}" ]; then
            metric=1
          else
            let metric+=1
          fi
        fi
      done

      if [ "${metric}" ]; then
        metric="metric ${metric}"
      fi

      if is_router_reachable ${gateway}; then
        logmessage "reason: ${reason}: \"ip -4 route replace ${route} ${metric}\""
        ip -4 route replace ${route} ${metric}

        if [ $? -ne 0 ]; then
          logmessage_err "failed to create static route: ${route} ${metric}"
        else
          route_destination="${route_destination} ${destination}"
          network_parameter_changed=1
        fi
      fi
    fi
  done
}


modify_static_routes()
{
  route_destination=""
  action=$1
  shift

  while [ $# -ne 0 ]; do
      if [ $# -lt 2 ]; then
        logmessage_err "number of ip addresses (pairs of destination and router) are not 2, see RFC2132 for details"
        break
      fi
      destination=$1
      gateway=$2
      shift; shift

      prefix="$( class_bits ${destination} )"

      metric=''
      for t in ${route_destination}; do
        if [ ${t} = ${destination} ]; then
          if [ -z "${metric}" ]; then
            metric=1
          else
            let metric+=1
          fi
        fi
      done

      if [ "${metric}" ]; then
        metric="metric ${metric}"
      fi

      if [ "${action}" = "delete" ]; then
        logmessage "reason: ${reason}: \"ip -4 route delete ${destination}/${prefix} via ${gateway} dev ${interface} ${metric}\""
        ip -4 route delete ${destination}/${prefix} via ${gateway} dev ${interface} ${metric}

        if [ $? -ne 0 ]; then
          logmessage_err "failed to delete static route: ${destination}/${prefix} via ${gateway} dev ${interface} ${metric}"
        else
          route_destination="${route_destination} ${destination}"
          network_parameter_changed=1
        fi
      elif is_router_reachable ${gateway}; then
        logmessage "reason: ${reason}: \"ip -4 route replace ${destination}/${prefix} via ${gateway} dev ${interface} ${metric}\""
        ip -4 route replace ${destination}/${prefix} via ${gateway} dev ${interface} ${metric}

        if [ $? -ne 0 ]; then
          logmessage_err "failed to create static route: ${destination}/${prefix} via ${gateway} dev ${interface} ${metric}"
        else
          route_destination="${route_destination} ${destination}"
          network_parameter_changed=1
        fi
      fi
  done
}

# General configuration function
dhconfig()
{
  if [ -z "${old_ip_address}" ]; then
    old_ip_address=`ip -4 addr show dev eth0 | grep inet | sed -e 's! *inet *!!' -e 's!/.*$!!'`
  fi

  if [ "${old_ip_address}" ] && [ "${alias_ip_address}" ] &&
     [ ! "${alias_ip_address}" = "${old_ip_address}" ]; then
    # possible new alias, remove old alias first
    logmessage "reason: ${reason}: \"ip -4 addr del ${old_ip_address} dev ${interface}:0\""
    ip -4 addr del ${old_ip_address} dev ${interface}:0
    network_parameter_changed=1
  fi

  if [ "${old_ip_address}" ] &&
     [ "${old_ip_address}" != "${new_ip_address}" ]; then
    # IP address changed.  Bringing down the interface will delete all
    # routes, and clear the ARP cache.
    logmessage "reason: ${reason}: \"ip -4 addr flush dev ${interface}\""
    ip -4 addr flush dev ${interface} >/dev/null 2>&1
    network_parameter_changed=1
  fi

  if [ "${reason}" = "BOUND" ] ||
     [ "${reason}" = "REBOOT" ] ||
     [ "${reason}" = "REUSE" ] ||
     [ "${old_ip_address}" != "${new_ip_address}" ] ||
     [ "${old_subnet_mask}" != "${new_subnet_mask}" ] ||
     [ "${old_network_number}" != "${new_network_number}" ] ||
     [ "${old_broadcast_address}" != "${new_broadcast_address}" ] ||
     [ "${old_routers}" != "${new_routers}" ] ||
     [ "${old_interface_mtu}" != "${new_interface_mtu}" ] ||
     [ "${old_static_routes}" != "${new_static_routes}" ] ||
     [ "${old_ms_classless_static_routes}" != "${new_ms_classless_static_routes}" ] ||
     [ "${old_classless_static_routes}" != "${new_classless_static_routes}" ]; then
    logmessage "reason: ${reason}: \"ip -4 addr add ${new_ip_address}/${new_prefix} broadcast ${new_broadcast_address} dev ${interface}\""
    ip -4 addr add ${new_ip_address}/${new_prefix} broadcast ${new_broadcast_address} dev ${interface} 2>/dev/null
    logmessage "reason: ${reason}: \"ip link set dev ${interface} up\""
    ip link set dev ${interface} up
    network_parameter_changed=1

    # The 576 MTU is only used for X.25 and dialup connections
    # where the admin wants low latency.  Such a low MTU can cause
    # problems with UDP traffic, among other things.  As such,
    # disallow MTUs from 576 and below by default, so that broken
    # MTUs are ignored, but higher stuff is allowed (1492, 1500, etc).
    if [ "${new_interface_mtu}" ] && [ ${new_interface_mtu} -gt 576 ]; then
      logmessage "reason: ${reason}: \"ip link set ${interface} mtu ${new_interface_mtu}\""
      ip link set ${interface} mtu ${new_interface_mtu}
      network_parameter_changed=1
    fi

    if [ -x ${ETCDIR_DHCP}/dhclient-${interface}-up-hooks ]; then
      . ${ETCDIR_DHCP}/dhclient-${interface}-up-hooks
    elif [ -x ${ETCDIR_DHCP}/dhclient-up-hooks ]; then
      . ${ETCDIR_DHCP}/dhclient-up-hooks
    fi

    # classless static routes
    # option classless-static-routes code 121 = array of unsigned integer 8;
    # option ms-classless-static-routes code 249 = array of unsigned integer 8;
    # e.g.: classless-static-routes=32 172 16 212 3 10 77 68 3 24 172 16 212 10 77 68 3

    # Remove old classless static routes
    if [ "${old_classless_static_routes}" ]; then
      modify_classless_static_routes delete ${old_classless_static_routes}
    fi

    # Remove old Microsoft classless static routes
    if [ "${old_ms_classless_static_routes}" ]; then
      modify_classless_static_routes delete ${old_ms_classless_static_routes}
    fi

    # Remove old static routes
    if [ "${old_static_routes}" ]; then
      IFS=', |' static_routes=${old_static_routes}
      modify_static_routes delete ${static_routes}
      IFS=' '
    fi

    # Routes
    if [ "${new_classless_static_routes}" ]; then
      # RFC3442: If the DHCP server returns both a Classless Static Routes option and
      #          a Router option, the DHCP client MUST ignore the Router option.
      new_routers=""
      modify_classless_static_routes replace ${new_classless_static_routes}

    elif [ -z "${new_classless_static_routes}" ] && [ "${new_ms_classless_static_routes}" ]; then
      # RFC3442: If the DHCP server returns both a Classless Static Routes option and
      #          a Router option, the DHCP client MUST ignore the Router option.
      #          Expect this also for MS option 249.
      new_routers=""
      modify_classless_static_routes replace ${new_ms_classless_static_routes}

    elif [ -z "${new_classless_static_routes}" ] && [ -z "${new_ms_classless_static_routes}" ] && [ "${new_static_routes}" ]; then
      # static routes
      # dhcp-options(5): option static-routes ip-address ip-address [, ip-address ip-address...];
      # But there is nothing defined to prefer static routes to routers option (RFC1533)
      IFS=', |' static_routes=${new_static_routes}
      modify_static_routes replace ${static_routes}
      IFS=' '
    fi

    # gateways
    if [ "${DEFROUTE}" != "no" -a \( -z "${DEFROUTE}" -o "${GATEWAYDEV}" = "${interface}" \) ]; then
      if [ -z "${GATEWAY}" -o \( "${DHCLIENT_IGNORE_GATEWAY}" -a \( "${DHCLIENT_IGNORE_GATEWAY:0:1}" = "y" -o "${DHCLIENT_IGNORE_GATEWAY:0:1}" = "Y" \) \) ]; then
        metric="${METRIC:-}"
        let i="${METRIC:-0}"
        default_routers=""

        for router in ${new_routers} ; do
          added_router=-

          for r in ${default_routers} ; do
            if [ "${r}" = "${router}" ]; then
              added_router=1
            fi
          done

          if [ -z "${router}" ] ||
             [ "${added_router}" = "1" ] ||
             [ $(IFS=. ip2num ${router}) -eq 0 ] ||
             [ "${router}" = "${new_broadcast_address}" -a "${new_subnet_mask}" != "255.255.255.255" ]; then
            continue
          fi

          default_routers="${default_routers} ${router}"
          add_default_gateway ${router} ${metric}
          let i=i+1
          metric=${i}
        done
      elif [ "${GATEWAY}" ]; then
        routersubnet="$( get_network_address ${GATEWAY} ${new_subnet_mask} )"
        mysubnet="$( get_network_address ${new_ip_address} ${new_subnet_mask} )"

        if [ "${routersubnet}" = "${mysubnet}" ]; then
          logmessage "reason: ${reason}: \"ip -4 route replace default via ${GATEWAY} dev ${interface}\" for mysubnet = \"${mysubnet}\""
          ip -4 route replace default via ${GATEWAY} dev ${interface}
          network_parameter_changed=1
        fi
      fi
    fi
  fi

  if [ ! "${new_ip_address}" = "${alias_ip_address}" ] &&
     [ "${alias_ip_address}" ]; then
    logmessage "IPv4 NETWORK PARAMETER CHANGED (L3)"
    logmessage "reason: ${reason}: \"ip -4 addr flush dev ${interface}:0\""
    ip -4 addr flush dev ${interface}:0 >/dev/null 2>&1
    logmessage "reason: ${reason}: \"ip -4 addr add ${alias_ip_address}/${alias_prefix} dev ${interface}:0\""
    ip -4 addr add ${alias_ip_address}/${alias_prefix} dev ${interface}:0
    logmessage "reason: ${reason}: \"ip -4 route replace ${alias_ip_address}/32 dev ${interface}:0\""
    ip -4 route replace ${alias_ip_address}/32 dev ${interface}:0
    network_parameter_changed=1
  fi

  if [ "${new_host_name}" ] && need_hostname; then
    hostname ${new_host_name} || echo "See -nc option in dhclient(8) man page."
  fi

  if [ "${DHCP_TIME_OFFSET_SETS_TIMEZONE}" ] &&
     [ "${DHCP_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "y" -o "${DHCP_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "Y" ]; then
    if [ "${new_time_offset}" ]; then
      # DHCP option "time-offset" is requested by default and should be
      # handled.  The geographical zone abbreviation cannot be determined
      # from the GMT offset, but the $ZONEINFO/Etc/GMT$offset file can be
      # used - note: this disables DST.
      let z=new_time_offset/3600
      if [ "${z}" -gt 0 ]; then hoursWest=$'+'${z}; else hoursWest=${z}; fi

      if [ "${hoursWest}" -lt 0 ]; then
        # tzdata treats negative 'hours west' as positive 'gmtoff'!
        let hoursWest*=-1
      fi

      tzfile=/usr/share/zoneinfo/Etc/GMT$(printf '%+d' ${hoursWest})
      if [ -e ${tzfile} ]; then
        save_previous ${ETCDIR}/localtime
        cp -fp ${tzfile} ${ETCDIR}/localtime
        touch ${ETCDIR}/localtime
      fi
    fi
  fi

  # execute any additional client side configuration scripts we have
  if [ -d ${ETCDIR_DHCP}/dhclient.d ]; then
    for f in ${ETCDIR_DHCP}/dhclient.d/*.sh ; do
      if [ -x ${f} ]; then
        subsystem="${f%.sh}"
        subsystem="${subsystem##*/}"
        . ${f}
        "${subsystem}_config"
      fi
    done
  fi

  # Provide the configured static routes for NetworkManager
  configured_static_routes=`ip -4 route show | grep -v src | sed -e 's!/! !' -e 's!via !!' -e 's!dev eth0 !!' -e 's!default!0.0.0.0 0!' -e 's! *$!,!g' | tr -d '\n'`
  echo "configured_static_routes=${configured_static_routes}" >>${tmpDhcpParameters}
}

#
# ### MAIN
#

if [ -x ${ETCDIR_DHCP}/dhclient-enter-hooks ]; then
  exit_status=0

  # dhclient-enter-hooks can abort dhclient-script by setting
  # the exit_status variable to a non-zero value
  . ${ETCDIR_DHCP}/dhclient-enter-hooks
  if [ ${exit_status} -ne 0 ]; then
    ${RM} ${tmpDhcpParameters}
    exit ${exit_status}
  fi
fi

new_prefix="$(get_prefix ${new_ip_address} ${new_subnet_mask})"
old_prefix="$(get_prefix ${old_ip_address} ${old_subnet_mask})"
alias_prefix="$(get_prefix ${alias_ip_address} ${alias_subnet_mask})"

# dhclient states:
#   EXPIRE -> PREINIT -> ARPSEND -> ARPCHECK -> BOUND

network_parameter_changed=0
case "${reason}" in
  MEDIUM)
    # Linux doesn't handle mediums (media)

    # Remove env file instead of calling netcmd, which is also a semaphore
    ${RM} ${tmpDhcpParameters}
    exit_with_hooks 0
    ;;

  PREINIT)
    if [ "${alias_ip_address}" ]; then
      # Bring down alias interface, its routes will disappear too.
      logmessage "reason: ${reason} \"ip link set ${interface}:0 down\""
      ip link set ${interface}:0 down
      network_parameter_changed=1
    fi

    # keep_old_ip is used for external forced renew
    if [ "${keep_old_ip}" = "yes" ]; then
      logmessage "reason: ${reason} \"ip link set ${interface} up\""
      if [ -r ${tmpKeepOldIpAddress} ]; then
        rm ${tmpKeepOldIpAddress}
      fi
    else
      logmessage "reason: ${reason} \"ip -4 addr flush dev ${interface}\""
      ip -4 addr flush dev ${interface} >/dev/null 2>&1
      logmessage "reason: ${reason} \"ip link set ${interface} up\""
    fi
    # Ensure interface is up
    ip link set ${interface} up
    network_parameter_changed=1

    if [ "${DHCLIENT_DELAY}" ] && [ ${DHCLIENT_DELAY} -gt 0 ]; then
      sleep ${DHCLIENT_DELAY}
    fi

    if [ ${network_parameter_changed} -eq 1 ]; then
      logmessage_warn "IPv4 NETWORK RECONFIGURED FOR DHCP (L3)"
    fi

    logmessage "netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}"
    netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}
    exit_with_hooks 0
    ;;

  ARPCHECK|ARPSEND)
    # This states aren't supported anymore since v3.0 by dhclient!
    if [ -z "${new_ip_address}" ] || [ -z "${interface}" ] || arping -q -f -c 2 -w 2 -D -I ${interface} ${new_ip_address}; then
      logmessage "No duplicate ip \"${new_ip_address}\" in reason \"${reason}\""
      # Remove env file instead of calling netcmd, which is also a semaphore
      ${RM} ${tmpDhcpParameters}
      exit_with_hooks 0
    else
      logmessage_warn "DUPLICATE IP: \"${new_ip_address}\" already in use, return 1 to dhclient"
      reason=DUPLICATEIP
      echo "reason=${reason}" >${tmpDhcpParameters}
      echo "old_ip_address=${old_ip_address}" >>${tmpDhcpParameters}

      deconfigureIpv4Interface
      logmessage_warn "IPv4 NETWORK DECONFIGURED (L3)"

      logmessage "netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}"
      netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}
      exit_with_hooks 1
    fi
    ;;

  BOUND|RENEW|REBIND|REBOOT|REUSE)
    if [ "${new_ip_address}" ] && [ "${interface}" ]; then
      if arping -q -f -c 2 -w 2 -D -I ${interface} ${new_ip_address}; then
        logmessage "No duplicate ip \"${new_ip_address}\" in reason \"${reason}\""
      else
        logmessage_warn "DUPLICATE IP: \"${new_ip_address}\" already in use, return 1 to dhclient"
        reason=DUPLICATEIP
        echo "reason=${reason}" >${tmpDhcpParameters}
        echo "old_ip_address=${old_ip_address}" >>${tmpDhcpParameters}

        if [ "${DHCLIENT_DELAY_DUPLICATE_IP}" ] && [ ${DHCLIENT_DELAY_DUPLICATE_IP} -gt 0 ]; then
          sleep ${DHCLIENT_DELAY_DUPLICATE_IP}
        fi
        deconfigureIpv4Interface
        logmessage_warn "IPv4 NETWORK DECONFIGURED (L3)"

        logmessage "netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}"
        netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}
        exit_with_hooks 1
      fi
    fi

    dhconfig

    if [ "${reason}" = "BOUND" ]; then
      # BOUND: The DHCP client has done an initial binding to a new address.

      # Check if lease reuse mode active and shutdown the watchdog
      if [ -f ${CONF_DIR_TMP}/addressWatchDog.pid ]; then
        logmessage_warn "IPv4 NETWORK LEASE OBTAINED - LEASE REUSE MODE DEACTIVATED (L3)"
        # if use expired lease watchdog is running kill it off
        watchdogPid=`cat ${CONF_DIR_TMP}/addressWatchDog.pid`
        kill -9 ${watchdogPid}
        ${RM} ${CONF_DIR_TMP}/addressWatchDog.pid
      fi
    fi

    if [ "${reason}" = "BOUND" -o "${reason}" = "RENEW" -o "${reason}" = "REBIND" ]; then
      # REBIND: The DHCP client has rebound to a new DHCP server.
      #         This can be handled as with RENEW, except that if the IP address has changed, the ARP table should be cleared.

      # RENEW: When a binding has been renewed, the script is called as in BOUND,
      #        except that in addition to all the variables starting with $new_, there is another set of variables starting with $old_.

      old_vlan=`cat ${tmpVlanFile} 2>/dev/null`
      if [ -f ${CONF_DIR}/enableDhcpv4LeaseReuse ]; then
        if [ "${old_vlan}" ]; then
          cp ${tmpDhcpParameters} ${dhcpv4BackupVlanConfigurationFile}
          echo "old_vlan=${old_vlan}" >>${dhcpv4BackupVlanConfigurationFile}
        else
          cp ${tmpDhcpParameters} ${dhcpv4BackupConfigurationFile}
        fi
      else
        if [ "${old_vlan}" ]; then
          cp ${tmpDhcpParameters} ${dhcpv4BackupVlanConfigurationTmpFile}
          echo "old_vlan=${old_vlan}" >>${dhcpv4BackupVlanConfigurationTmpFile}
        else
          cp ${tmpDhcpParameters} ${dhcpv4BackupConfigurationTmpFile}
        fi
      fi
      #Gratuitous arp to notify all network clients of new ip address
      arping -q -c 3 -A -I ${interface} ${new_ip_address} &
    fi

    if [ "${reason}" = "REUSE" ]; then
      logmessage_warn "NO DHCP SERVER - LEASE REUSE MODE ACTIVATED (L3)"
      /Opera_Deploy/networking/addressWatchDog.sh ${new_ip_address} ${interface} &

      #Gratuitous arp to notify all network clients of new ip address
      arping -q -c 3 -A -I ${interface} ${new_ip_address} &
    fi

    if [ ${network_parameter_changed} -eq 1 ]; then
      if [ \( "${reason}" = "BOUND" -o "${reason}" = "REBOOT" \) -a ${network_parameter_changed} -eq 1 ]; then
        logmessage "IPv4 NETWORK REINSTATED (L3)"
      elif  [ "${reason}" = "REUSE" ]; then
        logmessage "IPv4 NETWORK PARAMETER REUSED (L3)"
      else
        logmessage "IPv4 NETWORK PARAMETER CHANGED (L3)"
      fi
      log_ipv4_interface_status
    fi

    # Only for debugging
    # cp ${tmpDhcpParameters} ${tmpDhcpParameters}.${reason}.exit

    logmessage "netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}"
    netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}
    exit_with_hooks 0
    ;;

  EXPIRE|FAIL|RELEASE|STOP)
    if [ ! -f ${CONF_DIR}/enableDhcpv4LeaseReuse ] || [ \( "${reason}" = "RELEASE" -o "${reason}" = "STOP" \) -a -f ${CONF_DIR}/enableDhcpv4LeaseReuse ]; then

      if [ "${DHCP_TIME_OFFSET_SETS_TIMEZONE}" ] &&
         [ "${DHCP_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "y" -o "${DHCP_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "Y" ]; then
        # Only cleanup config files if there are no other dhclient processes running
        any_other_clients="$(ps | grep dhclient | grep -v grep | grep -v ${pid})"
        if [ -z "${any_other_clients}" ]; then
          if [ -e ${SAVE_DIR}/localtime.predhclient.${interface} ]; then
            ${RM} ${ETCDIR}/localtime
            cp ${SAVE_DIR}/localtime.predhclient.${interface} ${ETCDIR}/localtime
            ${RM} ${SAVE_DIR}/localtime.predhclient.${interface}
            touch ${ETCDIR}/localtime
            network_parameter_changed=1
          fi
        fi
      fi
    fi

    # execute any additional client side configuration scripts we have
    if [ -d ${ETCDIR_DHCP}/dhclient.d ]; then
      for f in ${ETCDIR_DHCP}/dhclient.d/*.sh ; do
        if [ -x ${f} ]; then
          subsystem="${f%.sh}"
          subsystem="${subsystem##*/}"
          . ${f}
          "${subsystem}_restore"
        fi
      done
    fi

    if [ -x ${ETCDIR_DHCP}/dhclient-${interface}-down-hooks ]; then
      . ${ETCDIR_DHCP}/dhclient-${interface}-down-hooks
    elif [ -x ${ETCDIR_DHCP}/dhclient-down-hooks ]; then
      . ${ETCDIR_DHCP}/dhclient-down-hooks
    fi

    if [ "${reason}" = "EXPIRE" ]; then
      # EXPIRE: The DHCP client has failed to renew its lease or acquire a new one, and the lease has expired.
      #         The IP address must be relinquished, if no dhcp reuse is configured.
      # Check if Dhcpv4LeaseReuse is enabled and skip flush in PREINIT
      if [ -f ${CONF_DIR}/enableDhcpv4LeaseReuse ]; then
        echo "keep_old_ip=yes" >${tmpKeepOldIpAddress}
        logmessage_warn "DHCP LEASE EXPIRED (L3), but try to reuse"
      else
        logmessage_warn "DHCP LEASE EXPIRED (L3)"
      fi

    elif [  "${reason}" = "FAIL" ]; then
      # FAIL: The DHCP client has been unable to contact any DHCP servers, and any leases that have been tested have not proved to be valid.
      #       NM decides if lease will be reused
      logmessage_warn "DHCP FAIL (L3), possible DHCP reused"

    elif [ "${reason}" = "RELEASE" ]; then
      # RELEASE: The dhclient has been executed using the -r flag, indicating that the administrator wishes it to release its lease(s).

      logmessage "reason: ${reason}: \"ip -4 addr flush dev ${interface}\""
      ip -4 addr flush dev ${interface} >/dev/null 2>&1

    elif [ "${reason}" = "STOP" ]; then
      # STOP:    The dhclient has been informed to shut down gracefully
      deconfigureIpv4Interface
      if [ ${network_parameter_changed} -eq 1 ]; then
        logmessage_warn "IPv4 NETWORK DECONFIGURED (L3)"
      fi
    fi

    if [ "${reason}" = "RELEASE" -o "${reason}" = "STOP" ]; then
      old_vlan=`cat ${tmpVlanFile} 2>/dev/null`
      if [ "${old_vlan}" ]; then
        ${RM} ${dhcpv4BackupVlanConfigurationFile}
      else
        ${RM} ${dhcpv4BackupConfigurationFile}
      fi
    fi

    logmessage "netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}"
    netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}
    exit_with_hooks 0
    ;;

  TIMEOUT)
    # TIMEOUT: The DHCP client has been unable to contact any DHCP servers.
    #          However, an old lease has been identified, and its parameters have been passed in as with BOUND.
    logmessage_warn "DHCP SERVER NOT REACHABLE (L3)"

    if [ "${new_routers}" ]; then
      if [ "${alias_ip_address}" ]; then
        logmessage "reason: ${reason}: \"ip -4 addr flush dev ${interface}:0\""
        ip -4 addr flush dev ${interface}:0 >/dev/null 2>&1
        network_parameter_changed=1
      fi

      if ping -q -c 1 -w 10 -I ${interface} $1; then
        dhconfig

        # Remove env file instead of calling netcmd, which is also a semaphore
        ${RM} ${tmpDhcpParameters}
        exit_with_hooks 0
      fi

      logmessage "reason: ${reason}: \"ip -4 addr flush dev ${interface}\""
      ip -4 addr flush dev ${interface} >/dev/null 2>&1
      # Do not set link down, because IPv6 may be need them
      # logmessage "reason: ${reason}: \"ip link set ${interface} down\""
      # ip link set ${interface} down
      network_parameter_changed=1

      logmessage "netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}"
      netcmd dhcpv4ConfChange -r ${reason} -f ${tmpDhcpParameters}
      exit_with_hooks 1
    else
      # Remove env file instead of calling netcmd, which is also a semaphore
      ${RM} ${tmpDhcpParameters}
      exit_with_hooks 1
    fi
    ;;

  *)
    logmessage_warn "unhandled state: ${reason}"

    # Remove env file instead of calling netcmd, which is also a semaphore
    ${RM} ${tmpDhcpParameters}
    exit_with_hooks 1
    ;;
esac

exit_with_hooks 0
