#!/bin/sh
#
# dhclient-script: Network interface configuration script run by
#                  dhclient based on DHCP client communication
#
# Copyright (C) 2008, 2009, 2010  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author(s): David Cantrell <dcantrell@redhat.com>
#
# ----------
# This script is a rewrite/reworking on dhclient-script originally
# included as part of dhcp-970306:
# dhclient-script for Linux. Dan Halbert, March, 1997.
# Updated for Linux 2.[12] by Brian J. Murrell, January 1999.
# Modified by David Cantrell <dcantrell@redhat.com> for Fedora and RHEL
# http://pkgs.fedoraproject.org/gitweb/?p=dhcp.git;a=commitdiff;h=fe8e8224b5cbd10550a02c6427f388cc34b4274e#patch2
# Siemens Enterprise Sven Jakob: make script busybox v1.15.3 compliant
# ----------
#
# CONF_DIR=/data/networking
# CONF_DIR_TMP=/tmp/networking
# SAVE_DIR=${CONF_DIR}/dhclient
#
# Files in CONF_DIR_TMP
# dhcpv6.eth0.env     ... dhcpv6 assigned values, written by dhcpv6Event.sh and read/removed by NM
#                         Lease reuse: is a copy from dhcpv6.eth0.env.backup or dhcpv6.eth0.env.vlan.backup
# netv6.env           ... manual assigned values, written by WNI used by netv4.sh
# dhcpv6.eth0.status  ... information how the IPv6 IFC is finally configured => syslog
# portDescr           ... not supported, can give a hint if phone is plugged to a different access switch port (PORT_TYPE_MAC_ADDRESS)
# vlan                ... assigned vlan-id, finally saved in *.env.vlan.backup
# keepOldIpv6Address  ... used to prevent a flush of the IFC, e.g. if it contains "keep_old_ip=yes", flush in PREINIT6 is skipped
#
# Files in CONF_DIR (persistent):
# dhcpv6.eth0.env.backup      ... backup of dhcpv6.eth0.env used for dhcp lease reuse from untagged vlan
# dhcpv6.eth0.env.vlan.backup ... backup of dhcpv6.eth0.env used for dhcp lease reuse from tagged vlan
# portDescr                   ... not supported, can give a hint if phone is plugged to a different access switch port (PORT_TYPE_MAC_ADDRESS)
# enableDhcpv6LeaseReuse      ... if file exists, then the feature is switched on

export PATH=./:/bin:/sbin:/usr/bin:/Opera_Deploy:/data/networking

scriptname=`basename $0`

exec 1>>/tmp/logs/${scriptname}.out
exec 2>>/tmp/logs/${scriptname}.out
# set -x # only for debug purpose

LOGFACILITY="daemon"

logmessage()
{
    msg="${1}"
    logger -p ${LOGFACILITY}.notice -t "NET6 [$$]" "dhclient: ${msg}"
}

logmessage_warn()
{
    msg="${1}"
    logger -p ${LOGFACILITY}.warning -t "NET6 [$$]" "dhclient: ${msg}"
}

logmessage_err()
{
    msg="${1}"
    logger -p ${LOGFACILITY}.err -t "NET6 [$$]" "dhclient: ${msg}"
}

if [ -z ${interface} ]; then
    interface=eth0
fi

ETCDIR="/etc/dhcp"

DHCLIENT_DELAY=1
GATEWAYDEV=${interface}
DHCLIENT_DELAY=1
PEERDNS="yes"
SEARCH=""
RES_OPTIONS=""

CONF_DIR=/data/networking
CONF_DIR_TMP=/tmp/networking
SAVE_DIR=${CONF_DIR}/dhclient

if [ ! -d ${CONF_DIR} ]; then
    mkdir -p ${CONF_DIR}
fi

if [ ! -d ${CONF_DIR_TMP} ]; then
    mkdir -p ${CONF_DIR_TMP}
fi

tmpDhcpParameters=${CONF_DIR_TMP}/dhcpv6.${interface}.env.$$
dhcpv6BackupConfigurationFile=${CONF_DIR}/dhcpv6.${interface}.env.backup
dhcpv6BackupVlanConfigurationFile=${CONF_DIR}/dhcpv6.${interface}.env.vlan.backup
tmpInterfaceStatus=${CONF_DIR_TMP}/dhcpv6.${interface}.status
tmpVlanFile=${CONF_DIR_TMP}/vlan
tmpKeepOldIpAddress=${CONF_DIR_TMP}/keepOldIpv6Address

# Import common definitions and functions
. /Opera_Deploy/networking/lib.sh

# Dhcp lease reuse configuration
if [ "$1" = "start" ]; then
  # Set environment for configuring the interface
  if [ "$2" ] && [ -r $2 ]; then
    . $2
    # Force reason to fully configure ifc
    reason=BOUND6
  else
    logmessage_warn "missing configuration for dhcpv6 lease reuse"
  fi
fi

recover_link_local

populate_global_gateway()
{
  for i in 1 2 3 4 5 6 7 8 9 10; do
    sleep 1 # give the last call to addr add time to have populated the global gw via neighbour disocery
    IFS=' '
    line=`ip -6 route show | grep 'default via'`
    if [ "${line}" ]; then
      set ${line}
      IFS=' '
      logmessage_warn "global gw found($i) '$3'"
      export new_ipv6AddressGlobalGateway=$3
      break;
    else
      IFS=' '
      logmessage_warn "global gw NOT found($i)"
    fi
  done
}

populate_global_gateway

# Prevent read race condition in NetworkManager,
delay_count=5
while [ -f ${tmpDhcpParameters} -a ${delay_count} -gt 0 ]; do
  let delay_count-=1;
  sleep 1
done

if [ ${delay_count} -le 0 ]; then
  # NetworkManager did not remove env file, reason may be a broken communication
  logmessage_warn "WARNING: reason: ${reason}: communication problem to NetworkManager!"
  # Only for debugging
  # cp ${tmpDhcpParameters} ${tmpDhcpParameters}.fatal
  rm -f ${tmpDhcpParameters}
fi

if [ -r ${tmpKeepOldIpAddress} ]; then
  # Set keep_old_ip to prevent unnecessary flush in case of external forced renew
  . ${tmpKeepOldIpAddress}
fi

save_env()
{
  logmessage "reason: ${reason}: saving env to $tmpDhcpParameters"
  env >${tmpDhcpParameters}
}

save_env

# Only for debugging
# cp ${tmpDhcpParameters} ${tmpDhcpParameters}.${reason}


exit_with_hooks()
{
  exit_status="${1}"

  if [ -x ${ETCDIR}/dhclient-exit-hooks ]; then
    . ${ETCDIR}/dhclient-exit-hooks
  fi

  exit ${exit_status}
}

### sysctl.h: NET_IPV6_OPT_DAD_ENABLE not included in kernel 2.6.20.19
### ip cmd does not support FLAG-LIST := [ FLAG-LIST ] FLAG
###  "FLAG := [ permanent | dynamic | secondary | primary | tentative | deprecated ]"

# Section 18.1.8. (Receipt of Reply Messages) of RFC 3315 says:
# The client SHOULD perform duplicate address detection on each of
# the addresses in any IAs it receives in the Reply message before
# using that address for traffic.
add_ipv6_addr_with_DAD()
{
  logmessage "reason: ${reason}: \"ip -6 addr add ${new_ip6_address}/${new_ip6_prefixlen} dev ${interface} scope global\""
  ip -6 addr add ${new_ip6_address}/${new_ip6_prefixlen} dev ${interface} scope global 2>/dev/null
  logmessage "reason: ${reason}: \"ip link set dev ${interface} up\""
  ip link set dev ${interface} up

  # repeatedly test whether newly added address passed
  # duplicate address detection (DAD)
  for i in 1 2 3 4 5; do
    sleep 1 # give the DAD some time

    # tentative flag = DAD is still not complete or failed
    ### REMARK: additional ip -6 option "tentative" not supported in kernel 2.6.20.19!
    duplicate=$(ip -6 addr show dev ${interface} | grep tentative \
           | grep ${new_ip6_address}/${new_ip6_prefixlen})

    # if there's no tentative flag, address passed DAD
    if [ -z "${duplicate}" ]; then
      break
    fi
  done

  # if there's still tentative flag = address didn't pass DAD = it's duplicate = remove it
  if [ "${duplicate}" ]; then
    logmessage_warn "DUPLICATE ADDRESS DETECTED ${new_ip6_address}/${new_ip6_prefixlen} (L3)"
    logmessage "reason: ${reason}: \"ip -6 addr del ${new_ip6_address}/${new_ip6_prefixlen} dev ${interface}\""
    ip -6 addr del ${new_ip6_address}/${new_ip6_prefixlen} dev ${interface}

    # Remove env file instead of calling netcmd, which is also a semaphore
    rm -f ${tmpDhcpParameters}
    exit_with_hooks 3
  else
    network_parameter_changed=1
  fi
}

dh6config()
{
  case "${reason}" in
    BOUND6)
      if [ -z "${new_ip6_address}" ] && [ -z "${new_ip6_prefixlen}" ]; then
        # Remove env file instead of calling netcmd, which is also a semaphore
        rm -f ${tmpDhcpParameters}
        exit_with_hooks 2
      fi

      add_ipv6_addr_with_DAD

      ;;

    RENEW6|REBIND6)
      if [ "${new_ip6_prefixlen}" ] &&
        [ "${new_ip6_address}" ] &&
        [ ! "${new_ip6_address}" = "${old_ip6_address}" ]; then
        add_ipv6_addr_with_DAD
      fi

      ;;

     DEPREF6)
      # The dhclient-script should then do ...something... to encourage applications
      # to move off the address and onto one of the remaining 'preferred' addresses.
      if [ -z "${new_ip6_prefixlen}" ]; then
        # Remove env file instead of calling netcmd, which is also a semaphore
        rm -f ${tmpDhcpParameters}
        exit_with_hooks 2
      fi

      ### ip: unknown command change ... "preferred_lft" is garbage
      ### so it is not possible to set the ipv6 address to "deprecated" = "preferred_lft 0"
      logmessage "cmd: \"ip -6 addr change ${new_ip6_address}/${new_ip6_prefixlen} dev ${interface} scope global preferred_lft 0\" not supported"
      ;;
  esac

  # execute any additional client side configuration scripts we have
  if [ -d ${ETCDIR}/dhclient.d ]; then
    for f in ${ETCDIR}/dhclient.d/*.sh ; do
      if [ -x ${f} ]; then
        subsystem="${f%.sh}"
        subsystem="${subsystem##*/}"
        . ${f}
        "${subsystem}_config"
      fi
    done
  fi
}

log_interface_status()
{
  echo "Configured IPv6 addresses on eth0:" >${tmpInterfaceStatus}
  ip -6 addr show dev eth0 | grep inet6 >>${tmpInterfaceStatus}
  echo "Configured IPv6 routes on eth0:" >>${tmpInterfaceStatus}
  ip -6 route show dev eth0 >>${tmpInterfaceStatus}
  logger -p ${LOGFACILITY}.notice -t "NET6 [$$]" <${tmpInterfaceStatus}
}

#
# ### MAIN
#

if [ -x ${ETCDIR}/dhclient-enter-hooks ]; then
  exit_status=0

  # dhclient-enter-hooks can abort dhclient-script by setting
  # the exit_status variable to a non-zero value
  . ${ETCDIR}/dhclient-enter-hooks
  if [ ${exit_status} -ne 0 ]; then
    exit ${exit_status}
  fi
fi

network_parameter_changed=0
case "${reason}" in
  MEDIUM)
    # Linux doesn't handle mediums (media)

    # Remove env file instead of calling netcmd, which is also a semaphore
    rm -f ${tmpDhcpParameters}
    exit_with_hooks 0
    ;;

  PREINIT6)
    # keep_old_ip is used for external forced renew
    if [ "${keep_old_ip}" = "yes" ]; then
      if [ -r ${tmpKeepOldIpAddress} ]; then
        rm ${tmpKeepOldIpAddress}
      fi
    else
      # Remove any stale addresses from aborted clients, but keep link local
      ### REMARK: ip: "permanent" is garbage
      logmessage "reason: ${reason}: \"ip -6 addr flush dev ${interface}\""
      ip -6 addr flush dev ${interface}
      if [ "${new_ip6_address_link_local}" -a "${new_ip6_prefixlen_link_local}" ]; then
        logmessage "reason: ${reason}: \"ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0\""
        ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0
      fi
    fi
    # Ensure interface is up
    ip link set ${interface} up

    logmessage_warn "IPv6 NETWORK RECONFIGURED FOR DHCP (L3)"

    logmessage "netcmd dhcpv6ConfChange -r ${reason} -f ${tmpDhcpParameters}"
    netcmd dhcpv6ConfChange -r ${reason} -f ${tmpDhcpParameters}
    exit_with_hooks 0
    ;;

  ARPCHECK|ARPSEND)
    logmessage_warn "reason: ${reason}: not supported"

    # Remove env file instead of calling netcmd, which is also a semaphore
    rm -f ${tmpDhcpParameters}
    exit_with_hooks 0
    ;;

  BOUND6|RENEW6|REBIND6|DEPREF6)
    dh6config

    if [ "${reason}" = BOUND6 -a ${network_parameter_changed} -eq 1 ]; then
      logmessage "IPv6 NETWORK REINSTATED (L3)"
    elif [ ${network_parameter_changed} -eq 1 ]; then
      logmessage "IPv6 NETWORK PARAMETER CHANGED (L3)"
    fi
    log_ipv6_interface_status

    if [ -f ${CONF_DIR}/enableDhcpv6LeaseReuse ]; then
      old_vlan=`cat ${tmpVlanFile} 2>/dev/null`
      if [ "${old_vlan}" ]; then
        cp ${tmpDhcpParameters} ${dhcpv6BackupVlanConfigurationFile}
        echo "old_vlan=${old_vlan}" >>${dhcpv6BackupVlanConfigurationFile}
      else
        cp ${tmpDhcpParameters} ${dhcpv6BackupConfigurationFile}
      fi
    fi

    logmessage "netcmd dhcpv6ConfChange -r ${reason} -f ${tmpDhcpParameters}"
    netcmd dhcpv6ConfChange -r ${reason} -f ${tmpDhcpParameters}
    exit_with_hooks 0
    ;;

  EXPIRE6|RELEASE6|STOP6)
    # Only restore config files if there are no other dhclient processes running
    # [ToDo]: prevent cleanup, if lease reuse mode


    if [ -z "${old_ip6_address}" ] || [ -z "${old_ip6_prefixlen}" ]; then
      # Remove env file instead of calling netcmd, which is also a semaphore
      rm -f ${tmpDhcpParameters}
      exit_with_hooks 2
    fi

    if [ "${reason}" = EXPIRE6 ]; then
      # EXPIRE6: The DHCP client has failed to renew its lease or acquire a new one, and the lease has expired.
      # The IP address must be relinquished, if no dhcp reuse is configured.

      logmessage_warn "DHCPv6 SERVER NOT REACHABLE (L3)"

      # For IPv6 dhcp lease reuse old_ip6_address must not be removed
      # [ToDo]: Not implemented yet
      if [ "${old_ip6_address}" ]; then
        logmessage "reason: ${reason}: \"ip -6 addr flush dev ${interface}\""
        ip -6 addr flush dev ${interface}
        if [ "${new_ip6_address_link_local}" -a "${new_ip6_prefixlen_link_local}" ]; then
          logmessage "reason: ${reason}: \"ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0\""
          ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0
        fi
      fi

      logmessage_warn "IPv6 NETWORK DECONFIGURED FOR GLOBAL (L3)"
    elif [ "${reason}" = RELEASE6 ]; then
      # RELEASE6: The dhclient has been executed using the -r flag, indicating that the administrator wishes it to release its lease(s).
      # dhclient calls this script before it has "XMT: Release on eth0", so a "ip -6 addr flush" here disturb the release
      # let do the "ip -6 addr flush" by PREINIT6 or by "net6.sh start"
      logmessage_warn "IPv6 NETWORK DECONFIGURED FOR GLOBAL (L3) postponed"
    elif [ "${reason}" = STOP6 ]; then
      # STOP6:  The dhclient has been informed to shut down gracefully

      logmessage "reason: ${reason}: \"ip -6 addr flush dev ${interface}\""
      ip -6 addr flush dev ${interface}
      if [ "${new_ip6_address_link_local}" -a "${new_ip6_prefixlen_link_local}" ]; then
        logmessage "reason: ${reason}: \"ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0\""
        ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0
      fi

      logmessage_warn "IPv6 NETWORK DECONFIGURED FOR GLOBAL (L3)"
    fi

    # execute any additional client side configuration scripts we have
    if [ -d ${ETCDIR}/dhclient.d ]; then
      for f in ${ETCDIR}/dhclient.d/*.sh ; do
        if [ -x ${f} ]; then
          subsystem="${f%.sh}"
          subsystem="${subsystem##*/}"
          . ${f}
          "${subsystem}_restore"
        fi
      done
    fi

    if [ -x ${ETCDIR}/dhclient-${interface}-down-hooks ]; then
      . ${ETCDIR}/dhclient-${interface}-down-hooks
    elif [ -x ${ETCDIR}/dhclient-down-hooks ]; then
      . ${ETCDIR}/dhclient-down-hooks
    fi

    logmessage "netcmd dhcpv6ConfChange -r ${reason} -f ${tmpDhcpParameters}"
    netcmd dhcpv6ConfChange -r ${reason} -f ${tmpDhcpParameters}
    exit_with_hooks 0
    ;;

  *)
    logmessage_warn "unhandled state: ${reason}"

    # Remove env file instead of calling netcmd, which is also a semaphore
    rm -f ${tmpDhcpParameters}
    exit_with_hooks 1
    ;;
esac

exit_with_hooks 0
