#!/bin/sh
#
# netv4.sh: Network interface configuration script run by
#           manual configuration
#
# Copyright (C) 2008, 2009, 2010  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author(s): David Cantrell <dcantrell@redhat.com>
#
# ----------
# This script is a rewrite/reworking on dhclient-script originally
# included as part of dhcp-970306:
# dhclient-script for Linux. Dan Halbert, March, 1997.
# Updated for Linux 2.[12] by Brian J. Murrell, January 1999.
# Modified by David Cantrell <dcantrell@redhat.com> for Fedora and RHEL
# http://pkgs.fedoraproject.org/gitweb/?p=dhcp.git;a=commitdiff;h=fe8e8224b5cbd10550a02c6427f388cc34b4274e#patch2
# Siemens Enterprise Sven Jakob: make script busybox v1.15.3 compliant
#                                add handling for dhcp reuse, syslog, etc.
# ----------

export PATH=./:/bin:/sbin:/usr/bin:/Opera_Deploy:/data/networking

scriptname=`basename $0`

exec 1>>/tmp/logs/${scriptname}.out
exec 2>>/tmp/logs/${scriptname}.out
# set -x # only for debug purpose

LOGFACILITY="daemon"

logmessage()
{
  msg="${1}"
  logger -p ${LOGFACILITY}.notice -t "NET [$$]" "${scriptname}: ${msg}"
}

logmessage_warn()
{
  msg="${1}"
  logger -p ${LOGFACILITY}.warning -t "NET [$$]" "${scriptname}: ${msg}"
}

logmessage_err()
{
  msg="${1}"
  logger -p ${LOGFACILITY}.err -t "NET [$$]" "${scriptname}: ${msg}"
}

ETCDIR="/Opera_Deploy/networking"

MANUAL_CONF_DELAY=1
MANUAL_CONF_DELAY_DUPLICATE_IP=10
GATEWAYDEV=${interface}
PEERDNS="yes"
SEARCH=""
RES_OPTIONS=""
CONF_DIR=/data/networking
SAVEDIR=${CONF_DIR}/manual
TMPDIR=/tmp/networking

if [ ! -d ${TMPDIR} ]; then
  mkdir -p ${TMPDIR}
fi

tmpParameters=${TMPDIR}/netv4.env
tmpKeepOldIpv4Address=${TMPDIR}/keepOldIpv4Address
nfsKeepIpv4Address=${CONF_DIR}/keepOldIpv4Address

# Only for debugging using a well defined parameter file
# cp ${tmpParameters}.save ${tmpParameters}

if [ "$1" = "start" ]; then
  # Set environment for configuring the interface
  if [ "$2" ] && [ -r $2 ]; then
    . $2
    tmpParameters=$2
  elif [ -r ${tmpParameters} ]; then
    . ${tmpParameters}
  else
    logmessage_warn "missing configuration \"${tmpParameters}\" for \"${interface}\""
  fi
fi

if [ -r ${tmpKeepOldIpv4Address} ]; then
  . ${tmpKeepOldIpv4Address}
fi

if [ -r ${nfsKeepIpv4Address} ]; then
  . ${nfsKeepIpv4Address}
fi

if [ -z "$interface" ]; then
  interface="eth0"
fi

tmpInterfaceStatus=${TMPDIR}/netv4.${interface}.status

# Import common definitions and functions
. /Opera_Deploy/networking/lib.sh

# Only for debugging
#if [ -r ${tmpParameters} ]; then
#  cp ${tmpParameters} ${tmpParameters}.backup
#fi

exit_with_hooks()
{
  exit_status="${1}"

  if [ -x ${ETCDIR}/netv4-exit-hooks ]; then
    . ${ETCDIR}/netv4-exit-hooks
  fi

  rm -f ${tmpParameters}
  exit ${exit_status}
}

quad2num()
{
  if [ $# -eq 4 ]; then
    let n="${1} << 24 | ${2} << 16 | ${3} << 8 | ${4}"
    echo "${n}"
    return 0
  else
    echo "0"
    return 1
  fi
}

# Result is a decimal number representing the hexadecimal value of the IP address
# ${1} = 10.77.68.2 => returns 172835842 (0xA4D4402)
ip2num()
{
  IFS="." quad2num ${1}
}

# Result is a doted IP address calculated from a decimal number representing the hexadecimal value of the IP address
# ${1}=172835842 (0xA4D4402) => returns 10.77.68.2
num2ip()
{
  let n="${1}"
  let o1="(n >> 24) & 0xff"
  let o2="(n >> 16) & 0xff"
  let o3="(n >> 8) & 0xff"
  let o4="n & 0xff"
  echo "${o1}.${o2}.${o3}.${o4}"
}

# Result is network address by given IP address and network mask or prefix
# e.g.: ${1}=10.77.68.2, ${2}=255.255.0.0 => returns 10.77.0.0 (network subnet address)
get_network_address()
{
  ip="${1}"
  nm="${2}"

  if [ "${ip}" ] && [ "${nm}" ]; then
    if [ `echo "${nm}" | grep '.*\..*'` ]; then
      ipcalc -s -n ${ip} ${nm} | cut -d '=' -f 2
    else
      ipcalc -s -n ${ip}/${nm} | cut -d '=' -f 2
    fi
  fi
}

# Result is prefix by given IP address and network mask
# e.g.: ${1}=10.77.68.2, ${2}=255.255.0.0 => returns 16 (result is a prefix)
get_prefix()
{
  ip="${1}"
  nm="${2}"

  if [ "${ip}" ] && [ "${nm}" ]; then
    ipcalc -s -p ${ip} ${nm} | cut -d '=' -f 2
  fi
}

# Result is the prefix (class bits) from a given (network) address
# ${1}=129.210.177.0 => returns 24 (result is a prefix)
class_bits()
{
  let ip=$(IFS='.' ip2num $1)

  if [ ${ip} -eq 0 ]; then
    echo 0
    return 0
  fi

  let bits=32
  let mask='255'
  while true; do
    let v='ip&mask'
    if [ "$v" -eq 0 ] ; then
      let bits-=8
    else
      break
    fi
    let 'mask<<=8'
  done
  echo $bits
}

is_router_reachable()
{
  # handle if the given router is not on our subnet
  router="${1}"
  routersubnet="$(get_network_address ${router} ${new_subnet_mask})"
  mysubnet="$(get_network_address ${new_ip_address} ${new_subnet_mask})"
  unreachable=0

  if [ ! "${routersubnet}" = "${mysubnet}" ]; then
    unreachable=1
    if arping -f -q -I ${interface} -w2 ${router}; then
      ip -4 route replace ${router}/32 dev ${interface}
      if [ $? -eq 0 ]; then
        ip -4 route delete ${router}/32 dev ${interface} >/dev/null 2>&1
        network_parameter_changed=1
        unreachable=0
      else
        logmessage_err "failed to create route for router ${router} not on subnet ${mysubnet}"
      fi
    else
      unreachable=1
      logmessage "Given router ${router} is unreachable on subnet ${mysubnet} router subnet ${routersubnet}"
    fi
  fi

  return ${unreachable}
}

add_default_gateway()
{
  router="${1}"
  metric=""

  if [ $# -gt 1 ] && [ ${2} -gt 0 ]; then
    metric="metric ${2}"
  fi

  if is_router_reachable ${router} ; then
    logmessage "reason: ${reason}: is_router_reachable: \"ip -4 route replace default via ${router} dev ${interface} ${metric}\""
    ip -4 route replace default via ${router} dev ${interface} ${metric}
    if [ $? -ne 0 ]; then
      logmessage_err "failed to create default route: ${router} dev ${interface} ${metric}"
      return 1
    else
      network_parameter_changed=1
      return 0
    fi
  fi

  return 1
}

start()
{
  if [ -x ${ETCDIR}/netv4-enter-hooks ]; then
    exit_status=0

    # netv4-enter-hooks can abort netv4-script by setting
    # the exit_status variable to a non-zero value
    . ${ETCDIR}/netv4-enter-hooks
    if [ ${exit_status} -ne 0 ]; then
      rm -f ${tmpParameters}
      exit ${exit_status}
    fi
  fi

  if [ "${alias_ip_address}" ]; then
  # Bring down alias interface, its routes will disappear too.
  logmessage "reason: ${reason} \"ip link set ${interface}:0 down\""
  ip link set ${interface}:0 down
  fi

  if [ "${keep_old_ip}" = "yes" ] || [ "${keep_old_ip}" = "nfs" ]; then
    logmessage "reason: ${reason} \"ip link set ${interface} up\""
    ip link set ${interface} up
  else
    logmessage "reason: ${reason} \"ip -4 addr flush dev ${interface}\""
    ip -4 addr flush dev ${interface} >/dev/null 2>&1
    logmessage "reason: ${reason} \"ip link set ${interface} up\""
    ip link set ${interface} up
  fi
  network_parameter_changed=1

  if [ "${MANUAL_CONF_DELAY}" ] && [ ${MANUAL_CONF_DELAY} -gt 0 ]; then
    sleep ${MANUAL_CONF_DELAY}
  fi

  if [ ${network_parameter_changed} -eq 1 ]; then
    logmessage_warn "IPv4 NETWORK RECONFIGURED FOR MANUAL CONFIGURATION (L3)"
  fi

  if [ "${new_ip_address}" ]; then
    if arping -q -f -c 2 -w 3 -D -I ${interface} ${new_ip_address}; then
      logmessage "Check for duplicate ip \"${new_ip_address}\" in reason \"${reason}\""
    else
      logmessage_warn "DUPLICATE IP: \"${new_ip_address}\" already in use, return 1 to enableInterface"

      if [ "${MANUAL_CONF_DELAY_DUPLICATE_IP}" ] && [ ${MANUAL_CONF_DELAY_DUPLICATE_IP} -gt 0 ]; then
        sleep ${MANUAL_CONF_DELAY_DUPLICATE_IP}
      fi
      exit_with_hooks 1
    fi
  fi

  if [ "${old_ip_address}" ] && [ "${alias_ip_address}" != "${old_ip_address}" ]; then
    # possible new alias, remove old alias first
    logmessage "reason: ${reason}: \"ip -4 addr del ${old_ip_address} dev ${interface}:0\""
    ip -4 addr del ${old_ip_address} dev ${interface}:0
    network_parameter_changed=1
  fi

  if [ "${old_ip_address}" ] && [ "${new_ip_address}" ] && [ "${old_ip_address}" != "${new_ip_address}" ]; then
    # IP address changed. Bringing down the interface will delete all
    # routes, and clear the ARP cache.
    logmessage "reason: ${reason}: \"ip -4 addr flush dev ${interface}\""
    ip -4 addr flush dev ${interface} >/dev/null 2>&1
    network_parameter_changed=1
  fi

  if [ "${old_ip_address}" != "${new_ip_address}" ] ||
    [ "${old_subnet_mask}" != "${new_subnet_mask}" ] ||
    [ "${old_network_number}" != "${new_network_number}" ] ||
    [ "${old_broadcast_address}" != "${new_broadcast_address}" ] ||
    [ "${old_classless_static_routes}" != "${new_classless_static_routes}" ] ||
    [ "${old_static_routes}" != "${new_static_routes}" ] ||
    [ "${old_routers}" != "${new_routers}" ] ||
    [ "${old_interface_mtu}" != "${new_interface_mtu}" ]; then

    new_prefix="$(get_prefix ${new_ip_address} ${new_subnet_mask})"

    if [ "${new_ip_address}" ] && [ "${new_subnet_mask}" ]; then
      if [ "${new_broadcast_address}" ]; then
        logmessage "reason: ${reason}: \"ip -4 addr add ${new_ip_address}/${new_prefix} broadcast ${new_broadcast_address} dev ${interface}\""
        ip -4 addr add ${new_ip_address}/${new_prefix} broadcast ${new_broadcast_address} dev ${interface}
      else
        logmessage "reason: ${reason}: \"ip -4 addr add ${new_ip_address}/${new_prefix} broadcast + dev ${interface}\""
        ip -4 addr add ${new_ip_address}/${new_prefix} broadcast + dev ${interface}
      fi
    fi

    logmessage "reason: ${reason}: \"ip link set dev ${interface} up\""
    ip link set dev ${interface} up
    network_parameter_changed=1

    # The 576 MTU is only used for X.25 and dialup connections
    # where the admin wants low latency. Such a low MTU can cause
    # problems with UDP traffic, among other things. As such,
    # disallow MTUs from 576 and below by default, so that broken
    # MTUs are ignored, but higher stuff is allowed (1492, 1500, etc).
    if [ "${new_interface_mtu}" ] && [ ${new_interface_mtu} -gt 576 ]; then
      logmessage "reason: ${reason}: \"ip link set ${interface} mtu ${new_interface_mtu}\""
      ip link set ${interface} mtu ${new_interface_mtu}
      network_parameter_changed=1
    fi

    if [ -x ${ETCDIR}/netv4-${interface}-up-hooks ]; then
      . ${ETCDIR}/netv4-${interface}-up-hooks
    elif [ -x ${ETCDIR}/netv4-up-hooks ]; then
      . ${ETCDIR}/netv4-up-hooks
    fi

    # New Server options, check documentation in dhcp-options(5) if this is valid:
    #  option classless-static-routes 25.129.210.177.132 192.168.0.113, 7.2 dhcp-24-240.brq.redhat.com, 0 192.168.0.113;
    #  option static-routes 129.210.177.132 192.168.0.113, 129.210.177.0 192.168.0.113
    # classless static routes must include also the default route see RFC3442 page 5
    #  A "0 10.77.68.2" is expanded to 0.0.0.0/0 => "ip -4 route replace 0.0.0.0/0 via 10.77.68.2 dev eth0" creates the deault route
    if [ "${new_classless_static_routes}" ] || [ "${new_static_routes}" ]; then
      if [ "${new_classless_static_routes}" ]; then
        IFS=', |' static_routes=${new_classless_static_routes}

        # If both a Classless Static Routes option and a Router option is set
        # the configuration ignores the Router option. (like DHCP RFC3442)
        new_routers=""
      else
        IFS=', |' static_routes=${new_static_routes}
      fi

      route_targets=""
      one_set_read=0
      for dest_descriptor_gateway in ${static_routes}; do
        if [ ${one_set_read} -eq 0 ]; then
          one_set_read=1
          dest_descriptor=${dest_descriptor_gateway}
          continue
        else
          one_set_read=0
          gateway=${dest_descriptor_gateway}
        fi

        if [ "${new_classless_static_routes}" ]; then
          prefix=$(echo ${dest_descriptor} | cut -d "." -f 1)
          target=$(echo ${dest_descriptor} | cut -d "." -f 2-)

          # target padding
          # Count No. of elements without wc or awk
          IFS='.'
          number_of_elements=0
          for i in ${target}; do
            let 'number_of_elements+=1'
          done
          IFS=''

          # pad target with ".0" if necessary
          let 'number_of_elements=4-number_of_elements'
          while [ ${number_of_elements} -gt 0 ]; do
            target=${target}".0"
            let number_of_elements-=1
          done

          # Have to zero any bits in the subnet number where the corresponding bit in the mask is zero.
          # In other words, the subnet number installed in the routing table is the logical AND of
          # the subnet number and subnet mask given in the Classless Static Routes option. (RFC3442)
          target="$( get_network_address ${target} ${prefix} )"
        else
          target=${dest_descriptor}
          prefix="$( class_bits ${target} )"
        fi

        metric=''
        for t in ${route_targets}; do
          if [ ${t} = ${target} ]; then
            if [ -z "${metric}" ]; then
              metric=1
            else
              let metric+=1
            fi
          fi
        done

        if [ "${metric}" ]; then
          metric="metric ${metric}"
        fi

        if is_router_reachable ${gateway}; then
          logmessage "reason: ${reason}: \"ip -4 route replace ${target}/${prefix} via ${gateway} dev ${interface} ${metric}\""
          ip -4 route replace ${target}/${prefix} via ${gateway} dev ${interface} ${metric}

          if [ $? -ne 0 ]; then
            logmessage_err "failed to create static route: ${target}/${prefix} via ${gateway} dev ${interface} ${metric}"
          else
            route_targets="${route_targets} ${target}"
            network_parameter_changed=1
          fi
        fi
      done
    fi

    # gateways (if classless static routes defined, the new_routers are deleted according to RFC3442 page 5
    if [ "${new_ip_address}" ] && [ "${new_subnet_mask}" ] && [ "${new_routers}" ]; then
      if [ "${DEFROUTE}" != "no" ] && [ -z "${GATEWAYDEV}" -o "${GATEWAYDEV}" = "${interface}" ]; then
        if [ -z "${GATEWAY}" -o \( "${DHCLIENT_IGNORE_GATEWAY}" -a \( "${DHCLIENT_IGNORE_GATEWAY:0:1}" = "y" -o "${DHCLIENT_IGNORE_GATEWAY:0:1}" = "Y" \) \) ]; then
          metric="${METRIC:-}"
          let i="${METRIC:-0}"
          default_routers=""

          for router in ${new_routers} ; do
            added_router=-

            for r in ${default_routers} ; do
              if [ "${r}" = "${router}" ]; then
                added_router=1
              fi
            done

            if [ -z "${router}" ] ||
              [ "${added_router}" = "1" ] ||
              [ $(IFS='.' ip2num ${router}) -eq 0 ] ||
              [ "${router}" = "${new_broadcast_address}" -a "${new_subnet_mask}" != "255.255.255.255" ]; then
              continue
            fi

            default_routers="${default_routers} ${router}"
            add_default_gateway ${router} ${metric}
            let i=i+1
            metric=${i}
          done
        elif [ "${GATEWAY}" ]; then
          routersubnet="$( get_network_address ${GATEWAY} ${new_subnet_mask} )"
          mysubnet="$( get_network_address ${new_ip_address} ${new_subnet_mask} )"

          if [ "${routersubnet}" = "${mysubnet}" ]; then
            logmessage "reason: ${reason}: \"ip -4 route replace default via ${GATEWAY} dev ${interface}\" for mysubnet = \"${mysubnet}\""
            ip -4 route replace default via ${GATEWAY} dev ${interface}
            network_parameter_changed=1
          fi
        fi
      fi
    fi
  fi

  if [ "${old_route1_dest}" != "${new_route1_dest}" ] ||
    [ "${old_route1_mask}" != "${new_route1_mask}" ] ||
    [ "${old_route1_gw}" != "${new_route1_gw}" ] ||
    [ "${old_route2_dest}" != "${new_route2_dest}" ] ||
    [ "${old_route2_mask}" != "${new_route2_mask}" ] ||
    [ "${old_route2_gw}" != "${new_route2_gw}" ]; then

    if [ "${new_route1_dest}" -a "${new_route1_mask}" -a "${new_route1_gw}" ]; then
      new_route1_prefix="$(get_prefix ${new_route1_dest} ${new_route1_mask})"
      if is_router_reachable ${new_route1_gw}; then
        logmessage "reason: ${reason}: \"ip -4 route replace ${new_route1_dest}/${new_route1_prefix} via ${new_route1_gw} dev ${interface}\""
        ip -4 route replace ${new_route1_dest}/${new_route1_prefix} via ${new_route1_gw} dev ${interface}
        network_parameter_changed=1
      else
        logmessage_warn "route1_gw \"${route1_gw}\" not reachable, static route not configured"
      fi
    fi

    if [ "${new_route2_dest}" -a "${new_route2_mask}" -a "${new_route2_gw}" ]; then
      new_route2_prefix="$(get_prefix ${new_route2_dest} ${new_route2_mask})"
      if is_router_reachable ${new_route2_gw}; then
        logmessage "reason: ${reason}: \"ip -4 route replace ${new_route2_dest}/${new_route2_prefix} via ${new_route2_gw} dev ${interface}\""
        ip -4 route replace ${new_route2_dest}/${new_route2_prefix} via ${new_route2_gw} dev ${interface}
        network_parameter_changed=1
      else
        logmessage_warn "route2_gw \"${route2_gw}\" not reachable, static route not configured"
      fi
    fi
  fi

  if [ ! "${new_ip_address}" = "${alias_ip_address}" ] &&
    [ "${alias_ip_address}" ]; then

    alias_prefix="$(get_prefix ${alias_ip_address} ${alias_subnet_mask})"

    logmessage "IPv4 NETWORK PARAMETER CHANGED (L3)"
    logmessage "reason: ${reason}: \"ip -4 addr flush dev ${interface}:0\""
    ip -4 addr flush dev ${interface}:0 >/dev/null 2>&1
    logmessage "reason: ${reason}: \"ip -4 addr add ${alias_ip_address}/${alias_prefix} dev ${interface}:0\""
    ip -4 addr add ${alias_ip_address}/${alias_prefix} dev ${interface}:0
    logmessage "reason: ${reason}: \"ip -4 route replace ${alias_ip_address}/32 dev ${interface}:0\""
    ip -4 route replace ${alias_ip_address}/32 dev ${interface}:0
    network_parameter_changed=1
  fi

  if [ "${new_host_name}" ] && need_hostname; then
    hostname ${new_host_name}
    network_parameter_changed=1
  fi

  if [ "${MANUAL_CONF_TIME_OFFSET_SETS_TIMEZONE}" ] &&
    [ "${MANUAL_CONF_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "y" -o "${MANUAL_CONF_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "Y" ]; then
    if [ "${new_time_offset}" ]; then
      # "time-offset" is requested by default and should be
      # handled. The geographical zone abbreviation cannot be determined
      # from the GMT offset, but the $ZONEINFO/Etc/GMT$offset file can be
      # used - note: this disables DST.
      let z=new_time_offset/3600
      if [ "${z}" -gt 0 ]; then hoursWest=$'+'${z}; else hoursWest=${z}; fi

      if [ "${hoursWest}" -lt 0 ]; then
        # tzdata treats negative 'hours west' as positive 'gmtoff'!
        let hoursWest*=-1
      fi

      tzfile=/usr/share/zoneinfo/Etc/GMT$(printf '%+d' ${hoursWest})
      if [ -e ${tzfile} ]; then
        save_previous /etc/localtime
        cp -fp ${tzfile} /etc/localtime
        touch /etc/localtime
        network_parameter_changed=1
      fi
    fi
  fi

  if [ ${network_parameter_changed} -eq 1 ]; then
    logmessage "IPv4 NETWORK REINSTATED (L3)"
    log_ipv4_interface_status
  fi
  exit_with_hooks 0
}


stop()
{
  # dhclient assigned value is preferred over manual, IPv6 is preferred over IPv4

  if [ "${MANUAL_CONF_TIME_OFFSET_SETS_TIMEZONE}" ] &&
     [ "${MANUAL_CONF_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "y" -o "${MANUAL_CONF_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "Y" ]; then
    if [ -e ${SAVEDIR}/localtime.premanual.${interface} ]; then
      rm -f /etc/localtime
      cp ${SAVEDIR}/localtime.premanual.${interface} /etc/localtime
      rm -f ${SAVEDIR}/localtime.premanual.${interface}
      touch /etc/localtime
      network_parameter_changed=1
    fi
  fi

  # remove dafault and static routes
  IFS='
  '
  line=`ip -4 route show dev eth0 | grep via`
  if [ "${line}" ]; then
    set ${line}
    IFS=''
    while [ $# -gt 3 ]
    do
      network=$1
      via=$2
      destination=$3
      if [ $via = "via" ];then
        ip -4 route delete $network via $destination dev eth0
      fi
      shift
    done
  else
    IFS=''
  fi

  if [ -x ${ETCDIR}/netv4-${interface}-down-hooks ]; then
    . ${ETCDIR}/netv4-${interface}-down-hooks
  elif [ -x ${ETCDIR}/netv4-down-hooks ]; then
    . ${ETCDIR}/netv4-down-hooks
  fi

  # Save alias_ip_address, old_ip_address, ... for deconfigureIpv4Interface
  old_ip_address=\"${new_ip_address}\"

  if [ -z "${old_ip_address}" ]; then
    # Be sure that the interface gets unconfigured in any way, if there is an unrecognized ip address set on interface
    IFS=' /'
    line=`ip -4 addr show dev eth0 | grep 'inet'`
    if [ "${line}" ]; then
      set ${line}
      IFS=''
      export old_ip_address=$2
      export old_ip_address_prefixlen=$3
    else
      IFS=''
    fi
  fi

  deconfigureIpv4Interface

  if [ ${network_parameter_changed} -eq 1 ]; then
    logmessage_warn "IPv4 NETWORK DECONFIGURED (L3)"
  fi

  exit_with_hooks 0
}


case "$1" in
  start)
    reason="start"
    start
    ;;
  stop)
    reason="stop"
    if [ "${keep_old_ip}" != "nfs" ]; then
      stop
    fi
    ;;
  *)
    echo $"Usage $0 {start|stop}"
    exit 1
    ;;
esac

exit_with_hooks 0
