#!/bin/sh
#
# netv4.sh: Network interface configuration script run by
#           manual configuration
#
# Copyright (C) 2008, 2009, 2010  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author(s): David Cantrell <dcantrell@redhat.com>
#
# ----------
# This script is a rewrite/reworking on dhclient-script originally
# included as part of dhcp-970306:
# dhclient-script for Linux. Dan Halbert, March, 1997.
# Updated for Linux 2.[12] by Brian J. Murrell, January 1999.
# Modified by David Cantrell <dcantrell@redhat.com> for Fedora and RHEL
# http://pkgs.fedoraproject.org/gitweb/?p=dhcp.git;a=commitdiff;h=fe8e8224b5cbd10550a02c6427f388cc34b4274e#patch2
# Siemens Enterprise Sven Jakob: make script busybox v1.15.3 compliant
#                                add handling for dhcp reuse, syslog, etc.
# ----------

export PATH=./:/bin:/sbin:/usr/bin:/Opera_Deploy:/data/networking

scriptname=`basename $0`

exec 1>>/tmp/logs/${scriptname}.out
exec 2>>/tmp/logs/${scriptname}.out
# set -x # only for debug purpose

LOGFACILITY="daemon"

logmessage()
{
  msg="${1}"
  logger -p ${LOGFACILITY}.notice -t "NET6 [$$]" "${scriptname}: ${msg}"
}

logmessage_warn()
{
  msg="${1}"
  logger -p ${LOGFACILITY}.warning -t "NET6 [$$]" "${scriptname}: ${msg}"
}

logmessage_err()
{
  msg="${1}"
  logger -p ${LOGFACILITY}.err -t "NET6 [$$]" "${scriptname}: ${msg}"
}

ETCDIR="/Opera_Deploy/networking"

MANUAL_CONF_DELAY=1
MANUAL_CONF_DELAY_DUPLICATE_IP=10
GATEWAYDEV=${interface}
PEERDNS="yes"
SEARCH=""
RES_OPTIONS=""
SAVEDIR=/data/networking/manual
TMPDIR=/tmp/networking

if [ ! -d ${TMPDIR} ]; then
  mkdir -p ${TMPDIR}
fi

tmpParameters=${TMPDIR}/netv6.env

# Only for debugging using a well defined parameter file
# cp ${tmpParameters}.save ${tmpParameters}

if [ "$1" = "start" ]; then
  # Get environment for configuring the interface
  if [ -r ${tmpParameters} ]; then
    . ${tmpParameters}
  else
    logmessage_warn "missing configuration \"${tmpParameters}\" for \"${interface}\""
  fi
fi

if [ -z "$interface" ]; then
  interface="eth0"
fi

tmpInterfaceStatus=${TMPDIR}/netv6.${interface}.status

# Import common definitions and functions
. /Opera_Deploy/networking/lib.sh

# Only for debugging
if [ -r ${tmpParameters} ]; then
  cp ${tmpParameters} ${tmpParameters}.backup
fi

publish_link_local()
{
  tmpParametersToNetworkManager=${TMPDIR}/dhcpv6.${interface}.env.$$

  # Prevent read race condition in NetworkManager,
  delay_count=5
  while [ -f ${tmpParametersToNetworkManager} -a ${delay_count} -gt 0 ]; do
    let delay_count-=1;
    sleep 1
  done

  if [ ${delay_count} -le 0 ]; then
    # NetworkManager did not remove env file, reason may be a broken communication
    logmessage_warn "WARNING: reason: ${reason}: communication problem to NetworkManager!"
    # Only for debugging
    # cp ${tmpParameters} ${tmpParameters}.fatal
    rm -f ${tmpParametersToNetworkManager}
  fi

  if [ "${new_ip6_address_link_local}" -a "${new_ip6_prefixlen_link_local}" ]; then
    echo 'reason=MANUAL' > ${tmpParametersToNetworkManager}
    echo "new_ip6_address_link_local=${new_ip6_address_link_local}" >> ${tmpParametersToNetworkManager}
    echo "new_ip6_prefixlen_link_local=${new_ip6_prefixlen_link_local}" >> ${tmpParametersToNetworkManager}
    logmessage "netcmd dhcpv6ConfChange -r \"MANUAL\" -f ${tmpParametersToNetworkManager}"
    netcmd dhcpv6ConfChange -r "MANUAL" -f ${tmpParametersToNetworkManager}
  fi
}

exit_with_hooks()
{
  exit_status="${1}"

  if [ -x ${ETCDIR}/netv6-exit-hooks ]; then
    . ${ETCDIR}/netv6-exit-hooks
  fi

  rm -f ${tmpParameters}
  exit ${exit_status}
}

is_router_reachable()
{
  # handle if the given router is not on our subnet
  router="${1}"
  unreachable=0

  if ping6 ${router} >/dev/null 2>&1;then
    ip -6 route replace ${router}/128 dev ${interface}
    if [ $? -eq 0 ]; then
      ip -6 route delete ${router}/128 dev ${interface} >/dev/null 2>&1
      network_parameter_changed=1
      unreachable=0
    else
      logmessage_err "is_router_reachable: failed to create route for router ${router} not on local link"
    fi
  else
    unreachable=1
    logmessage "is_router_reachable: Given router ${router} is unreachable on local link"
  fi

  return ${unreachable}
}

add_default_gateway()
{
  router="${1}"
  metric=""

  if [ $# -gt 1 ] && [ ${2} -gt 0 ]; then
    metric="metric ${2}"
  fi

  if is_router_reachable ${router}; then
    logmessage "reason: ${reason}: \"ip -6 route replace default via ${router} dev ${interface} ${metric}\""
    ip -6 route replace default via ${router} dev ${interface} ${metric}
    if [ $? -ne 0 ]; then
      logmessage_err "failed to create default route: ${router} dev ${interface} ${metric}"
      return 1
    else
      network_parameter_changed=1
      return 0
    fi
  fi

  return 1
}

### sysctl.h: NET_IPV6_OPT_DAD_ENABLE not included in kernel 2.6.20.19
### ip cmd does not support FLAG-LIST := [ FLAG-LIST ] FLAG
###  "FLAG := [ permanent | dynamic | secondary | primary | tentative | deprecated ]"

# Section 18.1.8. (Receipt of Reply Messages) of RFC 3315 says:
# The client SHOULD perform duplicate address detection on each of
# the addresses in any IAs it receives in the Reply message before
# using that address for traffic.
add_ipv6_addr_with_DAD()
{
  logmessage "reason: ${reason}: \"ip -6 addr add ${new_ip6_address}/${new_ip6_prefixlen} dev ${interface} scope global\""
  ip -6 addr add ${new_ip6_address}/${new_ip6_prefixlen} dev ${interface} scope global 2>/dev/null
  logmessage "reason: ${reason}: \"ip link set dev ${interface} up\""
  ip link set dev ${interface} up

  # repeatedly test whether newly added address passed
  # duplicate address detection (DAD)
  for i in 1 2 3 4 5; do
    sleep 1 # give the DAD some time

    # tentative flag = DAD is still not complete or failed
    ### REMARK: additional ip -6 option "tentative" not supported in kernel 2.6.20.19!
    duplicate=$(ip -6 addr show dev ${interface} | grep tentative \
         | grep ${new_ip6_address}/${new_ip6_prefixlen})

    # if there's no tentative flag, address passed DAD
    if [ -z "${duplicate}" ]; then
      break
    fi
  done

  # if there's still tentative flag = address didn't pass DAD = it's duplicate = remove it
  if [ "${duplicate}" ]; then
    logmessage_warn "DUPLICATE ADDRESS DETECTED ${new_ip6_address}/${new_ip6_prefixlen} (L3)"
    logmessage "reason: ${reason}: \"ip -6 addr del ${new_ip6_address}/${new_ip6_prefixlen} dev ${interface}\""
    ip -6 addr del ${new_ip6_address}/${new_ip6_prefixlen} dev ${interface}
    exit_with_hooks 3
  else
    network_parameter_changed=1
  fi
}

#
# ### MAIN
#
start()
{
  if [ -x ${ETCDIR}/netv6-enter-hooks ]; then
    exit_status=0

    # netv6-enter-hooks can abort netv6-script by setting
    # the exit_status variable to a non-zero value
    . ${ETCDIR}/netv6-enter-hooks
    if [ ${exit_status} -ne 0 ]; then
      rm -f ${tmpParameters}
      exit ${exit_status}
    fi
  fi

  network_parameter_changed=0

  if [ "${keep_old_ip}" = "yes" ]; then
    logmessage "reason: ${reason} \"ip link set ${interface} up\""
    ip link set ${interface} up
  else
    # remove any stale addresses from aborted configuration
    ### REMARK: ip: "permanent" is garbage
    logmessage "reason: ${reason}: \"ip -6 addr flush dev ${interface}\""
    ip -6 addr flush dev ${interface}
    if [ "${new_ip6_address_link_local}" -a "${new_ip6_prefixlen_link_local}" ]; then
      logmessage "reason: ${reason}: \"ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0\""
      ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0
    fi

    logmessage "reason: ${reason} \"ip link set ${interface} up\""
    ip link set ${interface} up
    network_parameter_changed=1
  fi

  if [ "${MANUAL_CONF_DELAY}" ] && [ ${MANUAL_CONF_DELAY} -gt 0 ]; then
    sleep ${MANUAL_CONF_DELAY}
  fi

  if [ ${network_parameter_changed} -eq 1 ]; then
   logmessage_warn "IPv6 NETWORK RECONFIGURED FOR MANUAL CONFIGURATION (L3)"
  fi

  if [ "${old_ip6_address}" != "${new_ip6_address}" ] ||
    [ "${old_ip6_prefixlen}" != "${new_ip6_prefixlen}" ]; then

    if [ "${new_ip6_address}" -a "${new_ip6_address}" != "::" -a "${new_ip6_prefixlen}" ]; then
      add_ipv6_addr_with_DAD
      network_parameter_changed=1
    fi
  fi

  if [ "${old_ip6_ipaddress_global_gw}" != "${new_ip6_ipaddress_global_gw}" ]; then
    if [ "${new_ip6_ipaddress_global_gw}" -a "${new_ip6_ipaddress_global_gw}" != "::" ]; then
      metric="${METRIC:-}"
      add_default_gateway ${new_ip6_ipaddress_global_gw} ${metric}
      network_parameter_changed=1
    fi
  fi

  if [ "${old_ip6_route1_dest}" != "${new_ip6_route1_dest}" ] ||
    [ "${old_ip6_route1_prefix}" != "${new_ip6_route1_prefix}" ] ||
    [ "${old_ip6_route1_gw}" != "${new_ip6_route1_gw}" ] ||
    [ "${old_ip6_route2_dest}" != "${new_ip6_route2_dest}" ] ||
    [ "${old_ip6_route2_prefix}" != "${new_ip6_route2_prefix}" ] ||
    [ "${old_ip6_route2_gw}" != "${new_ip6_route2_gw}" ]; then

    if [ "${new_ip6_route1_dest}" -a "${new_ip6_route1_prefix}" -a "${new_ip6_route1_gw}" ]; then
      if is_router_reachable ${new_ip6_route1_gw}; then
        logmessage "reason: ${reason}: \"ip -6 route replace ${new_ip6_route1_dest}/${new_ip6_route1_prefix} via ${new_ip6_route1_gw} dev ${interface}\""
        ip -6 route replace ${new_ip6_route1_dest}/${new_ip6_route1_prefix} via ${new_ip6_route1_gw} dev ${interface}
      else
        logmessage_warn "route1_gw \"${route1_gw}\" not reachable, static route not configured"
      fi
    fi

    if [ "${new_ip6_route2_dest}" -a "${new_ip6_route2_prefix}" -a "${new_ip6_route2_gw}" ]; then
      if is_router_reachable ${new_ip6_route2_gw}; then
        logmessage "reason: ${reason}: \"ip -6 route replace ${new_ip6_route2_dest}/${new_ip6_route2_prefix} via ${new_ip6_route2_gw} dev ${interface}\""
        ip -6 route replace ${new_ip6_route2_dest}/${new_ip6_route2_prefix} via ${new_ip6_route2_gw} dev ${interface}
      else
        logmessage_warn "route2_gw \"${route2_gw}\" not reachable, static route not configured"
      fi
    fi
  fi

  if [ "${new_host_name}" ] && need_hostname; then
    hostname ${new_host_name}
    network_parameter_changed=1
  fi

  if [ "${MANUAL_CONF_TIME_OFFSET_SETS_TIMEZONE}" ] &&
    [ "${MANUAL_CONF_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "y" -o "${MANUAL_CONF_TIME_OFFSET_SETS_TIMEZONE:0:1}" = "Y" ]; then
    if [ "${new_time_offset}" ]; then
      # "time-offset" is requested by default and should be
      # handled. The geographical zone abbreviation cannot be determined
      # from the GMT offset, but the $ZONEINFO/Etc/GMT$offset file can be
      # used - note: this disables DST.
      let z=new_time_offset/3600
      if [ "${z}" -gt 0 ]; then hoursWest=$'+'${z}; else hoursWest=${z}; fi

      if [ "${hoursWest}" -lt 0 ]; then
        # tzdata treats negative 'hours west' as positive 'gmtoff'!
        let hoursWest*=-1
      fi

      tzfile=/usr/share/zoneinfo/Etc/GMT$(printf '%+d' ${hoursWest})
      if [ -e ${tzfile} ]; then
        save_previous /etc/localtime
        cp -fp ${tzfile} /etc/localtime
        touch /etc/localtime
        network_parameter_changed=1
      fi
    fi
  fi

  if [ ${network_parameter_changed} -eq 1 ]; then
    logmessage "IPv6 NETWORK REINSTATED (L3)"
    log_ipv6_interface_status
  fi

  # execute any additional client side configuration scripts we have
  if [ -d ${ETCDIR}/netv6.d ]; then
    for f in ${ETCDIR}/netv6.d/*.sh ; do
      if [ -x ${f} ]; then
        subsystem="${f%.sh}"
        subsystem="${subsystem##*/}"
        . ${f}
        "${subsystem}_config"
      fi
    done
  fi
}


stop()
{
  # dhclient assigned value is preferred over manual, IPv6 is preferred over IPv4

  logmessage "reason: ${reason}: \"ip -6 addr flush dev ${interface}\""
  ip -6 addr flush dev ${interface}
  if [ "${new_ip6_address_link_local}" -a "${new_ip6_prefixlen_link_local}" ]; then
    logmessage "reason: ${reason}: \"ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0\""
    ip -6 addr add ${new_ip6_address_link_local}/${new_ip6_prefixlen_link_local} scope link dev eth0
  fi

  # remove dafault and static routes
  IFS='
  '
  line=`ip -6 route show dev eth0 | grep via`
  if [ "${line}" ]; then
    set ${line}
    IFS=''
    while [ $# -gt 3 ]
    do
     network=$1
     via=$2
     destination=$3
     if [ $via = "via" ];then
      ip -6 route delete $network via $destination dev eth0
     fi
     shift
    done
  else
    IFS=''
  fi

  logmessage_warn "IPv6 NETWORK DECONFIGURED FOR GLOBAL (L3)"

  # execute any additional client side configuration scripts we have
  if [ -d ${ETCDIR}/dhclient.d ]; then
    for f in ${ETCDIR}/dhclient.d/*.sh ; do
      if [ -x ${f} ]; then
        subsystem="${f%.sh}"
        subsystem="${subsystem##*/}"
        . ${f}
        "${subsystem}_restore"
      fi
    done
  fi

  if [ -x ${ETCDIR}/dhclient-${interface}-down-hooks ]; then
    . ${ETCDIR}/dhclient-${interface}-down-hooks
  elif [ -x ${ETCDIR}/dhclient-down-hooks ]; then
    . ${ETCDIR}/dhclient-down-hooks
  fi

  rm -f ${tmpParameters}

  exit_with_hooks 0
}

case "$1" in
  start)
    reason="start"
    recover_link_local
    publish_link_local
    start
    ;;
  stop)
    reason="stop"
    recover_link_local
    stop
    ;;
  *)
    echo "Usage "$0" {start|stop}"
    exit 1
    ;;
esac

exit_with_hooks 0
